@@ -42,18 +42,23 @@ static const char *replytarget = NULL;
 
 static FILE *fifo = NULL;
 static char fifopath[128] = "";
+static const char *ident = "sipwitch";
 
-size_t process::attach(const char *uid)
+size_t process::attach(const char *id, const char *uid)
 {
+	assert(id != NULL && *id != 0);
 	assert(uid != NULL && *uid != 0);
 
 	struct stat ino;
 
-	snprintf(fifopath, sizeof(fifopath), DEFAULT_VARPATH "/run/sipwitch");
+	if(!uid)
+		uid = process::identity();
+	ident = id;
+	snprintf(fifopath, sizeof(fifopath), DEFAULT_VARPATH "/run/%s", ident);
 	if(!stat(fifopath, &ino) && S_ISDIR(ino.st_mode) && !access(fifopath, W_OK)) 
-		snprintf(fifopath, sizeof(fifopath), DEFAULT_VARPATH "/run/sipwitch/control");
+		snprintf(fifopath, sizeof(fifopath), DEFAULT_VARPATH "/run/%s/control", ident);
 	else
-		snprintf(fifopath, sizeof(fifopath), "/tmp/sipwitch-%s/control", uid);
+		snprintf(fifopath, sizeof(fifopath), "/tmp/%s-%s/control", ident, uid);
 
 	remove(fifopath);
 	if(mkfifo(fifopath, 0660)) {
@@ -72,12 +77,12 @@ static void logfile(fsys_t &fs)
 {
 	char buf[128];
 
-	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/log/sipwitch.log");
+	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/log/%s.log", ident);
 	fsys::create(fs, buf, fsys::ACCESS_APPEND, 0660);
 	if(is(fs))
 		return;
 
-	snprintf(buf, sizeof(buf), "/tmp/sipwitch-%s/logfile", process::identity());
+	snprintf(buf, sizeof(buf), "/tmp/%s-%s/logfile", ident, process::identity());
 	fsys::create(fs, buf, fsys::ACCESS_APPEND, 0660); 
 }
 
@@ -191,23 +196,22 @@ static OVERLAPPED ovFifo;
 
 static void logfile(fsys_t& fd)
 {
-	assert(uid != NULL && *uid != 0);
-
 	char buf[256];
 	unsigned len;
 
 	GetEnvironmentVariable("APPDATA", buf, 192);
 	len = strlen(buf);
-	snprintf(buf + len, sizeof(buf) - len, "\\sipwitch\\service.log");
+	snprintf(buf + len, sizeof(buf) - len, "\\%s\\service.log", ident);
 	
 	fsys::create(fd, buf, fsys::ACCESS_APPEND, 0660);
 }
 
-size_t process::attach(const char *uid)
+size_t process::attach(const char *id, const char *uid)
 {
 	char buf[64];
 
-	snprintf(buf, sizeof(buf), "\\\\.\\mailslot\\sipwitch_ctrl");
+	ident = id;
+	snprintf(buf, sizeof(buf), "\\\\.\\mailslot\\%s_ctrl", ident);
 	hFifo = CreateMailslot(buf, 0, MAILSLOT_WAIT_FOREVER, NULL);
 	if(hFifo == INVALID_HANDLE_VALUE)
 		return 0;
@@ -392,11 +396,14 @@ bool process::system(const char *fmt, ...)
 
 	va_list args;
 	char buf[256];
-	
+
 	va_start(args, fmt);
 	if(fmt)
 		vsnprintf(buf, sizeof(buf), fmt, args);
 	va_end(args);
+
+	debug(5, "executing %s", buf);
+
 #ifdef	_MSWINDOWS_
 #else
 	int max = sizeof(fd_set) * 8;
@@ -465,7 +472,7 @@ bool process::control(const char *uid, const char *fmt, ...)
 
 	va_start(args, fmt);
 #ifdef	_MSWINDOWS_
-	snprintf(buf, sizeof(buf), "\\\\.\\mailslot\\sipwitch_ctrl");
+	snprintf(buf, sizeof(buf), "\\\\.\\mailslot\\%s_ctrl", ident);
 	fd = CreateFile(buf, GENERIC_WRITE, FILE_SHARE_READ, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 	if(fd == INVALID_HANDLE_VALUE)
 		return false;
@@ -474,10 +481,10 @@ bool process::control(const char *uid, const char *fmt, ...)
 	if(!uid)
 		uid = identity();
 
-	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/sipwitch/control");
+	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/%s/control", ident);
 	fd = ::open(buf, O_WRONLY | O_NONBLOCK);
 	if(fd < 0) {
-		snprintf(buf, sizeof(buf), "/tmp/sipwitch-%s/control", uid);
+		snprintf(buf, sizeof(buf), "/tmp/%s-%s/control", ident, uid);
 		fd = ::open(buf, O_WRONLY | O_NONBLOCK);
 	}
 	if(fd < 0)
@@ -495,6 +502,7 @@ bool process::control(const char *uid, const char *fmt, ...)
 	if(fd != hLoopback)
 		CloseHandle(fd);
 #else
+	buf[len] = 0;
 	if(::write(fd, buf, len) < len)
 		rtn = false;
 	::close(fd);
@@ -502,3 +510,118 @@ bool process::control(const char *uid, const char *fmt, ...)
 	return rtn;
 }
 
+bool process::state(const char *state)
+{
+	char buf[256], buf1[256];
+
+#ifdef	_MSWINDOWS_
+	return false;
+#else
+	snprintf(buf, sizeof(buf), DEFAULT_CFGPATH "/%s/%s.xml", ident, state);
+	if(!fsys::isfile(buf))
+		return false;
+	snprintf(buf1, sizeof(buf1), DEFAULT_VARPATH "/run/%s/state.xml", ident);
+	remove(buf1);
+	if(!stricmp(state, "up") || !stricmp(state, "none"))
+		return true;
+#ifdef	HAVE_SYMLINK
+	snprintf(buf1, sizeof(buf1), DEFAULT_VARPATH "/run/%s/state.xml", ident);
+	if(symlink(buf, buf1))
+		return false;
+#else
+	snprintf(buf1, sizeof(buf1), DEFAULT_VARPATH "/run/%s/state.xml", ident);
+	if(link(buf, buf1))
+		return false;
+#endif
+	return true;
+#endif
+}
+
+FILE *process::dumpfile(const char *uid)
+{
+	FILE *fp;
+	char buf[256];
+
+#ifdef	_MSWINDOWS_
+	GetEnvironmentVariable("APPDATA", buf, 192);
+	unsigned len = strlen(buf);
+	snprintf(buf + len, sizeof(buf) - len, "\\%s\\dumpfile.log", ident);	 
+#else
+	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/%s/dumpfile", ident);
+#endif
+	fp = fopen(buf, "w");
+#ifndef	_MSWINDOWS_
+	if(!fp) {
+		snprintf(buf, sizeof(buf), "/tmp/%s-%s/dumpfile", ident, uid);
+		fp = fopen(buf, "w");
+	}
+#endif
+	return fp;
+}
+
+FILE *process::snapshot(const char *uid)
+{
+	FILE *fp;
+	char buf[256];
+
+#ifdef	_MSWINDOWS_
+	GetEnvironmentVariable("APPDATA", buf, 192);
+	unsigned len = strlen(buf);
+	snprintf(buf + len, sizeof(buf) - len, "\\%s\\snapshot.log", ident);	 
+#else
+	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/%s/snapshot", ident);
+#endif
+	fp = fopen(buf, "w");
+#ifndef	_MSWINDOWS_
+	if(!fp) {
+		snprintf(buf, sizeof(buf), "/tmp/%s-%s/snapshot", ident, uid);
+		fp = fopen(buf, "w");
+	}
+#endif
+	return fp;
+}
+
+FILE *process::config(const char *uid)
+{
+	FILE *fp = NULL;
+	char buf[256];
+
+#ifdef _MSWINDOWS_
+	GetEnvironmentVariable("APPDATA", buf, 192);
+	unsigned len = strlen(buf);
+	snprintf(buf + len, sizeof(buf) - len, "\\%s\\config.xml", ident);
+	fp = fopen(buf, "r");
+	if(fp) {
+			process::errlog(DEBUG1, "loading config from %s", buf);
+			return fp;
+	}
+	GetEnvironmentVariable("USERPROFILE", buf, 192);
+	len = strlen(buf);
+	snprintf(buf + len, sizeof(buf) - len, "\\gnutelephony\\%s.xml", ident);
+#else
+	struct stat ino;
+
+	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/%s", ident);
+	if(uid && !stat(buf, &ino) && S_ISDIR(ino.st_mode)) {
+		snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/%s/config.xml", ident);
+		fp = fopen(buf, "r");
+		if(fp) {
+			process::errlog(DEBUG1, "loading config from %s", buf);
+			return fp;
+		}
+	}
+
+	if(uid)
+		snprintf(buf, sizeof(buf), DEFAULT_CFGPATH "/%s.conf", ident);
+	else
+		snprintf(buf, sizeof(buf), "%s/.%src", getenv("HOME"), ident); 
+#endif
+	process::errlog(DEBUG1, "loading config from %s", buf);
+	return fopen(buf, "r");
+}
+
+void process::util(const char *id)
+{
+	ident = id;
+}
+
