@@ -33,6 +33,7 @@ LinkedObject *service::callback::runlevels[4] = {NULL, NULL, NULL, NULL};
 unsigned service::callback::count = 0;
 condlock_t service::locking;
 service *service::cfg = NULL;
+volatile service::dialmode_t service::dialmode = service::ALL_DIALING;
 
 static char header[80] = "- welcome";
 static socket_t trap4 = INVALID_SOCKET;
@@ -113,7 +114,7 @@ void service::pointer::operator=(keynode *p)
 	node = p;
 }	
 
-service::callback::callback(int rl, const char *name) :
+service::callback::callback(int rl) :
 OrderedObject()
 {
 	crit(rl < (int)RUNLEVELS, "service runlevel invalid");
@@ -122,7 +123,6 @@ OrderedObject()
 		crit(rl > 0, "service runlevel invalid");
 	}
 	LinkedObject::enlist(&runlevels[rl]);
-	id = name;
 	active_flag = false;
 	runlevel = rl;
 	++count;
@@ -133,14 +133,6 @@ service::callback::~callback()
 	LinkedObject::delist(&runlevels[runlevel]);
 }
 
-void service::callback::activating(MappedRegistry *rr)
-{
-}
-
-void service::callback::expiring(MappedRegistry *rr)
-{
-}
-
 void service::callback::snapshot(FILE *fp)
 {
 }
@@ -150,14 +142,8 @@ bool service::callback::check(void)
 	return true;
 }
 
-bool service::callback::classifier(rtpproxy::session *sid, rtpproxy::session *src, struct sockaddr *addr)
+void service::callback::reload(service *keys)
 {
-	return false;
-}
-
-bool service::callback::reload(service *keys)
-{
-	return true;
 }
 
 void service::callback::start(service *keys)
@@ -363,6 +349,30 @@ service::keynode *service::list(const char *id)
 	return NULL;
 }
 
+service::keynode *service::getUser(const char *id)
+{
+	assert(id != NULL && *id != 0);
+	unsigned path;
+	linked_pointer<keymap> map;
+
+	if(!cfg)
+		goto bail;
+
+	locking.access();
+
+	path = NamedObject::keyindex(id, CONFIG_KEY_SIZE);
+	map = cfg->keys[path];
+	
+	while(map) {
+		if(!stricmp(map->id, id))
+			return map->node;
+		map.next();
+	}
+
+bail:
+	locking.release();
+	return NULL;
+}
 
 service::keynode *service::getProtected(const char *id)
 {
@@ -767,28 +777,13 @@ void service::dumpfile(const char *uid)
 {
 	assert(uid == NULL || *uid != 0);
 
-	FILE *fp;
-	char buf[256];
 	linked_pointer<callback> cb;
 	keynode *env = getEnviron();
 
 	if(!uid)
 		uid = getValue(env, "USER");
 
-#ifdef	_MSWINDOWS_
-	GetEnvironmentVariable("APPDATA", buf, 192);
-	unsigned len = strlen(buf);
-	snprintf(buf + len, sizeof(buf) - len, "\\sipwitch\\dumpfile.log");	 
-#else
-	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/sipwitch/dumpfile");
-#endif
-	fp = fopen(buf, "w");
-#ifndef	_MSWINDOWS_
-	if(!fp) {
-		snprintf(buf, sizeof(buf), "/tmp/sipwitch-%s/dumpfile", uid);
-		fp = fopen(buf, "w");
-	}
-#endif
+	FILE *fp = process::dumpfile(uid);
 
 	release(env);
 
@@ -805,40 +800,10 @@ void service::dumpfile(const char *uid)
 	fclose(fp);
 }
 
-void service::activate(MappedRegistry *rr)
-{
-	unsigned rl = 0;
-	linked_pointer<callback> cb;
-
-	while(rl < RUNLEVELS) {
-		cb = callback::runlevels[rl++];
-		while(is(cb)) {
-			cb->activating(rr);
-			cb.next();
-		}
-	}
-}
-
-void service::expire(MappedRegistry *rr)
-{
-	unsigned rl = 0;
-	linked_pointer<callback> cb;
-
-	while(rl < RUNLEVELS) {
-		cb = callback::runlevels[rl++];
-		while(cb) {
-			cb->expiring(rr);
-			cb.next();
-		}
-	}
-}
-
 void service::snapshot(const char *uid)
 {
 	assert(uid == NULL || *uid != 0);
 
-	FILE *fp;
-	char buf[256];
 	linked_pointer<callback> cb;
 	unsigned rl = 0;
 	keynode *env = getEnviron();
@@ -846,21 +811,7 @@ void service::snapshot(const char *uid)
 	if(!uid)
 		uid = getValue(env, "USER");
 
-#ifdef	_MSWINDOWS_
-	GetEnvironmentVariable("APPDATA", buf, 192);
-	unsigned len = strlen(buf);
-	snprintf(buf + len, sizeof(buf) - len, "\\sipwitch\\snapshot.log");	 
-#else
-	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/sipwitch/snapshot");
-#endif
-	fp = fopen(buf, "w");
-#ifndef _MSWINDOWS_
-	if(!fp) {
-		snprintf(buf, sizeof(buf), "/tmp/sipwitch-%s/snapshot", uid);
-		fp = fopen(buf, "w");
-	}
-#endif
-
+	FILE *fp = process::snapshot(uid);
 	release(env);
 
 	if(!fp) {
@@ -885,27 +836,8 @@ void service::snapshot(const char *uid)
 	process::errlog(DEBUG1, "snapshot completed");
 }
 
-bool service::confirm(const char *user)
+void service::confirm(const char *user)
 {
-	return true;
-}
-
-service::callback *service::getComponent(const char *id)
-{
-	assert(id != NULL && *id != 0);
-
-	linked_pointer<callback> cb;
-	unsigned rl = 0;
-
-	while(rl < RUNLEVELS) {
-		cb = callback::runlevels[rl++];
-		while(cb) {
-			if(cb->isActive() && cb->id && !stricmp(cb->id, id))
-				return *cb;
-			cb.next();
-		}
-	}
-	return NULL;
 }
 
 bool service::check(void)
@@ -924,69 +856,32 @@ bool service::check(void)
 	return rtn;
 }
 
-bool service::classify(rtpproxy::session *sid, rtpproxy::session *src, struct sockaddr *addr)
+void service::commit(const char *user)
 {
+	service *orig;
 	linked_pointer<callback> cb;
 	unsigned rl = 0;
-	bool rtn = false;
 
-	while(!rtn && rl < RUNLEVELS) {
+	while(rl < RUNLEVELS) {
 		cb = callback::runlevels[rl++];
-		while(!rtn && is(cb)) {
-			rtn = cb->classifier(sid, src, addr);
-			cb.next();
-		}
-	}
-	return rtn;
-}
-
-bool service::commit(const char *user)
-{
-	linked_pointer<callback> cb;
-	unsigned rl = 0;
-	bool rtn = true;
-
-	while(rtn && rl < RUNLEVELS) {
-		cb = callback::runlevels[rl++];
-		while(rtn && is(cb)) {
-			rtn = cb->reload(this);
+		while(is(cb)) {
+			cb->reload(this);
 			cb.next();
 		}
 	}
 
-	if(!rtn || !confirm(user))
-		return false;
+	confirm(user);
 
 	locking.modify();
-	if(cfg)
-		delete cfg;
+	orig = cfg;
 	cfg = this;
 	locking.commit();
-	return true;
-}
 
-bool service::state(const char *state)
-{
-	char buf[256];
-
-#ifdef	_MSWINDOWS_
-	return false;
-#else
-	snprintf(buf, sizeof(buf), DEFAULT_CFGPATH "/sipwitch/%s.xml", state);
-	if(!fsys::isfile(buf))
-		return false;
-	remove(DEFAULT_VARPATH "/run/sipwitch/state.xml");
-	if(!stricmp(state, "up") || !stricmp(state, "none"))
-		return true;
-#ifdef	HAVE_SYMLINK
-	if(symlink(buf, DEFAULT_VARPATH "/run/sipwitch/state.xml"))
-		return false;
-#else
-	if(link(buf, DEFAULT_VARPATH "/run/sipwitch/state.xml"))
-		return false;
-#endif
-	return true;
-#endif
+	// let short-term volatile references settle before we delete it...
+	if(orig) {
+		Thread::sleep(1000);
+		delete orig;
+	}
 }
 
 FILE *service::open(const char *uid, const char *cfgfile)
@@ -994,10 +889,6 @@ FILE *service::open(const char *uid, const char *cfgfile)
 	assert(cfgfile == NULL || *cfgfile != 0);
 	assert(uid == NULL || *uid != 0);
 
-	char buf[256];
-	struct stat ino;
-	FILE *fp;
-
 	if(!cfgfile)
 		cfgfile = getenv("CFG");
 
@@ -1006,36 +897,7 @@ FILE *service::open(const char *uid, const char *cfgfile)
 		return fopen(cfgfile, "r");
 	}
 
-#ifdef _MSWINDOWS_
-	GetEnvironmentVariable("APPDATA", buf, 192);
-	unsigned len = strlen(buf);
-	snprintf(buf + len, sizeof(buf) - len, "\\sipwitch\\config.xml");
-	fp = fopen(buf, "r");
-	if(fp) {
-			process::errlog(DEBUG1, "loading config from %s", buf);
-			return fp;
-	}
-	GetEnvironmentVariable("USERPROFILE", buf, 192);
-	len = strlen(buf);
-	snprintf(buf + len, sizeof(buf) - len, "\\gnutelephony\\sipwitch.xml");
-#else
-	snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/sipwitch");
-	if(uid && !stat(buf, &ino) && S_ISDIR(ino.st_mode)) {
-		snprintf(buf, sizeof(buf), DEFAULT_VARPATH "/run/sipwitch/config.xml");
-		fp = fopen(buf, "r");
-		if(fp) {
-			process::errlog(DEBUG1, "loading config from %s", buf);
-			return fp;
-		}
-	}
-
-	if(uid)
-		snprintf(buf, sizeof(buf), DEFAULT_CFGPATH "/sipwitch.conf");
-	else
-		snprintf(buf, sizeof(buf), "%s/.sipwitchrc", getenv("HOME")); 
-#endif
-	process::errlog(DEBUG1, "loading config from %s", buf);
-	return fopen(buf, "r");
+	return process::config(uid);
 }
 
 bool service::match(const char *digits, const char *match, bool partial)
