@@ -74,9 +74,11 @@ class __EXPORT MappedRegistry : public ReusableObject
 public:
 	char	userid[MAX_USERID_SIZE];
 	char	display[MAX_DISPLAY_SIZE];
+	char	remote[MAX_USERID_SIZE];
 	enum {OFFLINE = 0, IDLE, BUSY, AWAY, DND} status;
-	enum {EXPIRED = 0, USER, GATEWAY, SERVICE, REJECT, REFER, TEMPORARY} type;
+	enum {EXPIRED = 0, USER, GATEWAY, SERVICE, REJECT, REFER, TEMPORARY, REGISTRATION} type;
 	bool hidden;
+	unsigned rid;				// registry remap or peer id 
 	unsigned ext;				// 0 or extnum
 	unsigned count;				// active regs count
 	volatile unsigned inuse;	// in use for call count
