@@ -103,8 +103,8 @@ void registry::route::operator delete(void *obj)
 registry::registry() :
 service::callback(0), mapped_reuse<MappedRegistry>()
 {
-	realm = "Local Telephony";
-	digest = "MD5";
+	realm = (volatile char *)"Local Telephony";
+	digest = (volatile char *)"MD5";
 	prefix = 100;
 	range = 600;
 	expires = 300l;
@@ -349,6 +349,7 @@ void registry::expire(mapped *rr)
 	rr->ext = 0;
 	rr->status = MappedRegistry::OFFLINE;
 	rr->type = MappedRegistry::EXPIRED;
+	rr->rid = -1;
 	rp->delist(&keys[path]);
 	reg.removeLocked(rr);
 }
@@ -364,35 +365,35 @@ void registry::cleanup(time_t period)
 		rr = static_cast<mapped*>(reg.pos(regcount++));
 		locking.modify();
 		if(rr->type != MappedRegistry::EXPIRED && rr->expires && rr->expires + period < now && !rr->inuse) {
-			service::expire(rr);
+			server::expire(rr);
 			expire(rr);
 		}
 		else if(!rr->inuse && rr->type == MappedRegistry::EXPIRED && !rr->status == MappedRegistry::OFFLINE)
-			service::expire(rr);
+			server::expire(rr);
 		locking.commit();
 		Thread::yield();
 	}
 }
 
-bool registry::reload(service *cfg)
+void registry::reload(service *cfg)
 {
 	assert(cfg != NULL);
 
 	const char *key = NULL, *value;
 	linked_pointer<service::keynode> sp = cfg->getList("registry");
 
-	while(sp) {
+	while(is(sp)) {
 		key = sp->getId();
 		value = sp->getPointer();
 		if(key && value) {
 			if(!stricmp(key, "mapped") && !isConfigured()) 
 				mapped_entries = atoi(value);
 			else if(!stricmp(key, "digest") && !isConfigured()) {
-				digest = value;
-				digest.upper();
+				digest = cfg->dup(value);
+				String::upper((char *)digest);
 			}
 			else if(!stricmp(key, "realm") && !isConfigured())
-				realm = value;
+				realm = cfg->dup(value);
 			else if(!stricmp(key, "prefix") && !isConfigured())
 				prefix = atoi(value);
 			else if(!stricmp(key, "range") && !isConfigured())
@@ -408,7 +409,7 @@ bool registry::reload(service *cfg)
 	}
 
 	if(isConfigured())
-		return true;
+		return;
 
 	if(range) {
 		extmap = new mapped *[range];
@@ -424,8 +425,7 @@ bool registry::reload(service *cfg)
 	memset(contacts, 0, sizeof(LinkedObject *) * keysize);
 	memset(publishing, 0, sizeof(LinkedObject *) * keysize);
 	memset(addresses, 0, sizeof(LinkedObject *) * keysize);
-	process::errlog(INFO, "realm %s", realm.c_str());
-	return true;
+	process::errlog(INFO, "realm %s", realm);
 }
 
 unsigned registry::getEntries(void)
@@ -462,6 +462,7 @@ registry::mapped *registry::invite(const char *id)
 	rr->created = 0;
 	rr->display[0] = 0;
 	rr->inuse = 1;
+	rr->rid = -1;
 
 	String::set(rr->userid, sizeof(rr->userid), id);
 	rr->ext = 0;
@@ -508,6 +509,7 @@ registry::mapped *registry::create(const char *id)
 	server::getProvision(id, user);
 	node = user.keys;
 	cp = "none";
+	rr->rid = -1;
 	rr->type = MappedRegistry::EXPIRED;
 	rr->expires = 0;
 	rr->created = 0;
@@ -840,6 +842,32 @@ bool registry::exists(const char *id)
 	return rtn;
 }
 
+registry::mapped *registry::dialing(const char *id)
+{
+	assert(id != NULL && *id != 0);
+
+	mapped *rr;
+	unsigned ext = 0;
+
+	if(isExtension(id))
+		ext = atoi(id);
+
+	locking.access();
+	rr = find(id);
+
+	// if extension dialing, and we find by id but have ext #, then ignore
+	if(rr && service::dialmode == service::EXT_DIALING && rr->ext != 0)
+		rr = NULL;
+
+	// assuming not user id exclusive dialing, then we can try ext...
+	if(!rr && service::dialmode != service::USER_DIALING && reg.range && ext >= reg.prefix && ext < reg.prefix + reg.range)
+		rr = extmap[ext - reg.prefix];
+	if(!rr)
+		locking.release();
+	return rr;
+}
+
+
 registry::mapped *registry::access(const char *id)
 {
 	assert(id != NULL && *id != 0);
@@ -916,6 +944,7 @@ unsigned registry::mapped::setTarget(Socket::address& target_addr, time_t lease,
 			oi = ai;
 		memcpy(&tp->address, ai, len);
 		memcpy(&contact, oi, len);
+		stack::sipUserid(target_contact, remote, sizeof(remote));
 		if(creating) {
 			tp->index.registry = this;
 			tp->index.address = (struct sockaddr *)(&tp->address);
@@ -1099,6 +1128,7 @@ unsigned registry::mapped::addTarget(Socket::address& target_addr, time_t lease,
 		expired->enlist(&targets);
 		expired->status = registry::target::READY;
 		memcpy(&contact, oi, len);
+		stack::sipUserid(target_contact, remote, sizeof(remote));
 		if(origin)
 			delete origin;
 		++count;
@@ -1145,6 +1175,7 @@ unsigned registry::mapped::setTargets(Socket::address& target_addr)
 		tp = new target;
 		memcpy(&tp->address, al->ai_addr, len);
 		memcpy(&contact, &tp->address, len);
+		remote[0] = 0;
 		stack::getInterface((struct sockaddr *)(&tp->iface), (struct sockaddr *)(&tp->address));
 		stack::sipAddress(&tp->address, tp->contact, userid);
 		tp->expires = 0l;
