@@ -101,7 +101,7 @@ static bool activating(int argc, char **args)
 	if(!reg->setTargets(*addr))
 		rtn = false;
 	registry::detach(reg);
-	service::activate(reg);
+	server::activate(reg);
 	delete addr;
 	return rtn;
 }
@@ -115,6 +115,58 @@ service(id, PAGING_SIZE)
 	acl = NULL;
 }
 
+void server::authenticate(int id, const char *realm)
+{
+	linked_pointer<modules::sipwitch> cb = getModules();
+
+	while(is(cb)) {
+		cb->authenticate(id, realm);
+		cb.next();
+	}
+}
+
+void server::registration(int id, modules::regmode_t mode)
+{
+	linked_pointer<modules::sipwitch> cb = getModules();
+
+	while(is(cb)) {
+		cb->registration(id, mode);
+		cb.next();
+	}
+}
+
+void server::activate(MappedRegistry *rr)
+{
+	linked_pointer<modules::sipwitch> cb = getModules();
+
+	while(is(cb)) {
+		cb->activating(rr);
+		cb.next();
+	}
+}
+
+void server::expire(MappedRegistry *rr)
+{
+	linked_pointer<modules::sipwitch> cb = getModules();
+
+	while(is(cb)) {
+		cb->expiring(rr);
+		cb.next();
+	}
+}
+
+bool server::classify(rtpproxy::session *sid, rtpproxy::session *src, struct sockaddr *addr)
+{
+	linked_pointer<modules::sipwitch> cb = getModules();
+	bool rtn = false;
+
+	while(!rtn && is(cb)) {
+		rtn = cb->classifier(sid, src, addr);
+		cb.next();
+	}
+	return rtn;
+}
+
 unsigned server::forwarding(keynode *leaf)
 {
 	unsigned mask = 0;
@@ -177,7 +229,7 @@ bool server::create(const char *id, keynode *node)
 	return false;
 }
 
-bool server::confirm(const char *user)
+void server::confirm(const char *user)
 {
 	assert(user == NULL || *user != 0);
 
@@ -295,7 +347,7 @@ bool server::confirm(const char *user)
 	}
 
 	node = provision->getFirst();
-	while(node) {
+	while(is(node)) {
 		number = 0;
 		leaf = node->leaf("id");
 		id = NULL;
@@ -351,7 +403,6 @@ bool server::confirm(const char *user)
 		}
 		node.next();	
 	}
-	return true;
 }
 
 void server::release(keynode *node)
@@ -449,42 +500,6 @@ service::keynode *server::getConfig(void)
 	return (keynode *)cfg;
 }
 
-void server::getExtension(const char *id, usernode& user)
-{
-	assert(id != NULL && *id != 0);
-	assert(cfg != NULL);
-
-	unsigned ext = atoi(id);
-	unsigned range = registry::getRange();
-	unsigned prefix = registry::getPrefix();
-	server *cfgp;
-	keynode *node = NULL;
-	const char *cp;
-
-	server::release(user);
-
-	locking.access();
-	cfgp = static_cast<server *>(cfg);
-	if(!cfgp) {
-		locking.release();
-		return;
-	}
-
-	if(range && ext >= prefix && ext < prefix + range)
-		node = cfgp->extmap[ext - range];
-	if(!node) {
-		node = cfgp->find(id);
-		cp = NULL;
-		if(node)
-			cp = node->getPointer();
-		if(!cp || !stricmp(cp, "route") || !stricmp(cp, "gateway"))
-			node = NULL;
-	}
-	if(!node)
-		locking.release();
-	user.keys = node;
-}
-
 Socket::address *server::getContact(const char *uid)
 {
 	assert(uid != NULL && *uid != 0);
@@ -553,6 +568,40 @@ unsigned server::getForwarding(const char *uid)
 	return mask;
 }
 
+void server::getDialing(const char *uid, usernode& user)
+{
+	assert(uid != NULL && *uid != 0);
+	assert(cfg != NULL);
+
+	keynode *leaf = NULL;
+	keynode *node;
+	server *cfgp;
+	unsigned range = registry::getRange();
+	unsigned prefix = registry::getPrefix();
+	unsigned ext = atoi(uid);
+
+	server::release(user);
+
+	locking.access();
+	cfgp = static_cast<server*>(cfg);
+	if(!cfgp) {
+		locking.release();
+		return;
+	}
+	node = cfgp->find(uid);
+	if(node)
+		leaf = node->leaf("extension");
+	if(node && leaf && service::dialmode == service::EXT_DIALING)
+		node = NULL;
+
+	if(!node && service::dialmode != service::USER_DIALING && range && ext >= prefix && ext < prefix + range)
+		node = cfgp->extmap[ext - prefix];
+
+	if(!node)
+		locking.release();
+	user.keys = node;
+}
+
 void server::getProvision(const char *uid, usernode& user)
 {
 	assert(uid != NULL && *uid != 0);
@@ -635,7 +684,7 @@ void server::dump(FILE *fp)
 	fprintf(fp, "Server:\n");
 	fprintf(fp, "  allocated pages: %d\n", server::allocate());
 	fprintf(fp, "  configure pages: %d\n", cfg->getPages());
-	fprintf(fp, "  memory paging:   %d\n", PAGING_SIZE);
+	fprintf(fp, "  memory paging:   %ld\n", PAGING_SIZE);
 	keynode *reg = getPath("registry");
 	if(reg && reg->getFirst()) {
 		fprintf(fp, "  registry keys:\n");
@@ -666,8 +715,6 @@ void server::reload(const char *uid)
 #endif
 
 	server *cfgp = new server("sipwitch");
-
-	static server *reclaim = NULL;
 	
 	crit(cfgp != NULL, "reload without config");
 
@@ -685,23 +732,11 @@ void server::reload(const char *uid)
 			return;
 		}
 
-	if(!cfgp->commit(uid)) {
-		process::errlog(ERRLOG, "config rejected");
-		if(reclaim) {
-			locking.modify();
-			delete reclaim;
-			locking.commit();
-		}
-		reclaim = cfgp;
-	}
+	cfgp->commit(uid);
 	if(!cfg) {
 		process::errlog(FAILURE, "no configuration");
 		exit(2);
 	}
-	if(reclaim) {
-		delete reclaim;
-		reclaim = NULL;
-	}
 }
 
 unsigned server::allocate(void)
@@ -749,12 +784,12 @@ void server::plugins(const char *argv0, const char *list)
 		return;
 
 	if(!stricmp(list, "auto")) {
-		string::set(path, sizeof(path), argv0);
+		String::set(path, sizeof(path), argv0);
 		ep = strstr(path, LIB_PREFIX + 1);
 		if(ep)
 			ep[strlen(LIB_PREFIX)] = 0;
 		else 
-			string::set(path, sizeof(path), DEFAULT_LIBPATH "/sipwitch");
+			String::set(path, sizeof(path), DEFAULT_LIBPATH "/sipwitch");
 		el = strlen(path);
 		fsys::open(dir, path, fsys::ACCESS_DIRECTORY);
 		while(is(dir) && fsys::read(dir, buffer, sizeof(buffer)) > 0) {
@@ -769,14 +804,14 @@ void server::plugins(const char *argv0, const char *list)
 		fsys::close(dir);
 	}
 	else {
-		string::set(buffer, sizeof(buffer), list);
-		while(NULL != (cp = string::token(buffer, &tp, ", ;:\r\n"))) {
-			string::set(path, sizeof(path), argv0);
+		String::set(buffer, sizeof(buffer), list);
+		while(NULL != (cp = String::token(buffer, &tp, ", ;:\r\n"))) {
+			String::set(path, sizeof(path), argv0);
 			ep = strstr(path, LIB_PREFIX + 1);
 			if(ep) {
 				ep[strlen(LIB_PREFIX)] = 0;
-				string::add(path, sizeof(path), cp);
-				string::add(path, sizeof(path), DLL_SUFFIX);
+				String::add(path, sizeof(path), cp);
+				String::add(path, sizeof(path), DLL_SUFFIX);
 				if(fsys::isfile(path)) {
 					process::errlog(INFO, "loading %s" DLL_SUFFIX " locally", cp);
 					goto loader;
@@ -879,7 +914,7 @@ void server::run(const char *user)
 	if(dt->tm_year < 1900)
 		dt->tm_year += 1900;
 
-	process::printlog("server startup %04d-%02d-%02d %02d:%02d:%02d\n",
+	process::printlog("server starting %04d-%02d-%02d %02d:%02d:%02d\n",
 		dt->tm_year, dt->tm_mon + 1, dt->tm_mday,
 		dt->tm_hour, dt->tm_min, dt->tm_sec);
 
@@ -953,7 +988,7 @@ invalid:
 			if(argc != 2)
 				goto invalid;
 			state = String::unquote(argv[1], "\"\"\'\'()[]{}");
-			if(!service::state(state))
+			if(!process::state(state))
 				process::reply("invalid state");
 			fp = fopen(DEFAULT_VARPATH "/run/sipwitch/state.def", "w");
 			if(fp) {
