@@ -37,7 +37,6 @@
 NAMESPACE_SIPWITCH
 using namespace UCOMMON_NAMESPACE;
 
-#define	CONFIG_KEY_SIZE 177
 #define	PAGING_SIZE	(2048 * sizeof(void *))
 
 #define	ENABLE_ALLOW_INVITE		0x0001
@@ -84,16 +83,16 @@ public:
 		
 		void operator=(mapped *ptr);
 		
-		inline operator bool()
+		inline operator bool() const
 			{return entry != NULL;};
 
-		inline bool operator!()
+		inline bool operator!() const
 			{return entry == NULL;};
 
-		inline mapped *operator->()
+		inline mapped *operator->() const
 			{return entry;};
 
-		inline mapped *operator*()
+		inline mapped *operator*() const
 			{return entry;};
 	};
 
@@ -139,7 +138,7 @@ private:
 	};
 
 	bool check(void);
-	bool reload(service *cfg);
+	void reload(service *cfg);
 	void start(service *cfg);
 	void stop(service *cfg);
 	void snapshot(FILE *fp);
@@ -151,8 +150,8 @@ private:
 
 	volatile time_t expires;
 
-	String digest;
-	String realm;
+	volatile char *digest;
+	volatile char *realm;
 	unsigned prefix;
 	unsigned range;
 	unsigned routes;
@@ -161,10 +160,10 @@ public:
 	registry();
 
 	inline static const char *getRealm(void)
-		{return reg.realm;};
+		{return (const char *)reg.realm;};
 
 	inline static const char *getDigest(void)
-		{return reg.digest;};
+		{return (const char *)reg.digest;};
 
 	inline static time_t getExpires(void)
 		{return reg.expires;};
@@ -178,7 +177,6 @@ public:
 	inline static unsigned getRoutes(void)
 		{return reg.routes;};
 
-
 	static unsigned getEntries(void);
 	static unsigned getIndex(mapped *rr);
 	static bool isExtension(const char *id);
@@ -190,6 +188,7 @@ public:
 	static mapped *create(const char *id);
 	static mapped *access(const char *id);
 	static mapped *invite(const char *id);
+	static mapped *dialing(const char *id);
 	static bool exists(const char *id);
 	static pattern *getRouting(unsigned trs, const char *id);
 	static void detach(mapped *m);
@@ -253,10 +252,10 @@ private:
 		char secret[MAX_USERID_SIZE];
 		enum {NONE, DIGEST}	authtype;
 
-		inline bool isSource(void)
+		inline bool isSource(void) const
 			{return (this == parent->source);};
 
-		inline bool isTarget(void)
+		inline bool isTarget(void) const
 			{return (this == parent->target);};
 	};
 
@@ -332,7 +331,7 @@ private:
 		static void operator delete(void *obj);
 	};
 
-	bool reload(service *cfg);
+	void reload(service *cfg);
 	void start(service *cfg);
 	void stop(service *cfg);
 	void snapshot(FILE *fp);
@@ -409,14 +408,6 @@ class __LOCAL server : public service
 private:
 	typedef	linked_value<profile_t, LinkedObject> profile;
 
-	class __LOCAL keymap : public LinkedObject
-	{
-	public:
-		service::keynode *node;
-		const char *id;
-	};
-
-	LinkedObject *keys[CONFIG_KEY_SIZE];
 	cidr::policy *acl;
 	keynode **extmap;
 	keynode *provision;
@@ -425,7 +416,7 @@ private:
 	bool create(const char *id, keynode *node);
 	keynode *find(const char *id);
 
-	bool confirm(const char *user);
+	void confirm(const char *user);
 	void dump(FILE *fp);
 
 public:
@@ -435,7 +426,7 @@ public:
 	static profile_t *getProfile(const char *id); 
 	static keynode *getRouting(const char *id);
 	static void getProvision(const char *id, usernode& user);
-	static void getExtension(const char *id, usernode& user);
+	static void getDialing(const char *id, usernode& user);
 	static keynode *getConfig(void);
 	static unsigned getForwarding(const char *id);
 	static cidr *getPolicy(struct sockaddr *addr);
@@ -455,6 +446,15 @@ public:
 	static caddr_t allocate(size_t size, LinkedObject **list, volatile unsigned *count = NULL);
 	static unsigned allocate(void);
 	static void regdump(void);	
+
+	static bool classify(rtpproxy::session *session, rtpproxy::session *source, struct sockaddr *addr);
+	static void activate(MappedRegistry *rr);
+	static void expire(MappedRegistry *rr);
+	static void registration(int id, modules::regmode_t mode);
+	static void authenticate(int id, const char *realm);
+
+	static inline bool isProxied(void)
+		{return classify(NULL, NULL, NULL);};
 };
 
 class __LOCAL messages : public service::callback
@@ -476,7 +476,7 @@ private:
 	static messages manager;
 
 	bool check(void);
-	bool reload(service *cfg);
+	void reload(service *cfg);
 	void cleanup(void);
 	void snapshot(FILE *fp);
 
