@@ -153,7 +153,7 @@ void stack::background::run(void)
 		if(!signalled && timeout) {
 			if(timeout > interval) 
 				timeout = interval;
-			Conditional::wait(interval);
+			Conditional::wait(timeout);
 		}
 		timeout = expires.get();
 		if(signalled || !timeout) {
@@ -642,7 +642,7 @@ void stack::snapshot(FILE *fp)
 	locking.release();
 } 
 
-bool stack::reload(service *cfg)
+void stack::reload(service *cfg)
 {
 	assert(cfg != NULL);	
 
@@ -769,7 +769,6 @@ bool stack::reload(service *cfg)
 		hash = new LinkedObject*[keysize];
 		memset(hash, 0, sizeof(LinkedObject *) * keysize);
 	}
-	return true;
 }
 
 const char *stack::getScheme(void)
@@ -1032,7 +1031,7 @@ set:
 
 	if(ap && !ap->getList()) {
 		delete ap;
-		addr = NULL;
+		ap = NULL;
 	}
 	return ap;
 }
