@@ -30,7 +30,7 @@ static char *userid = NULL;
 static char *secret = NULL;
 static char *identity = NULL;
 
-class __LOCAL subscriber : private service::callback
+class __LOCAL subscriber : private modules::sipwitch
 {
 private:
 	unsigned short port;
@@ -56,7 +56,7 @@ public:
 	subscriber();
 
 	bool classifier(rtpproxy::session *session, rtpproxy::session *source, struct sockaddr *addr);
-	bool reload(service *cfg);
+	void reload(service *cfg);
 	void start(service *cfg);
 	void stop(service *cfg);
 	void snapshot(FILE *fp);
@@ -87,7 +87,7 @@ void subscriber::listener::run(void)
 }
 
 subscriber::subscriber() :
-service::callback(-1)
+modules::sipwitch()
 {
 	port = 9000;
 	count = 0; 	
@@ -126,7 +126,7 @@ void subscriber::snapshot(FILE *fp)
 	fprintf(fp, "subscriber:\n"); 
 } 
 
-bool subscriber::reload(service *cfg)
+void subscriber::reload(service *cfg)
 {
 	assert(cfg != NULL);	
 
@@ -192,7 +192,6 @@ bool subscriber::reload(service *cfg)
 		}
 		sp.next();
 	}
-	return true;
 }
 
 bool subscriber::classifier(rtpproxy::session *sid, rtpproxy::session *src, struct sockaddr *addr)
